SPCM_ERROR_ORIGIN_MASK = 0x80000000
SPCM_ERROR_ORIGIN_LOCAL = 0x00000000
SPCM_ERROR_ORIGIN_REMOTE = 0x80000000
ERR_OK = 0x0000
ERR_INIT = 0x0001
ERR_NR = 0x0002
ERR_TYP = 0x0003
ERR_FNCNOTSUPPORTED = 0x0004
ERR_BRDREMAP = 0x0005
ERR_KERNELVERSION = 0x0006
ERR_HWDRVVERSION = 0x0007
ERR_ADRRANGE = 0x0008
ERR_INVALIDHANDLE = 0x0009
ERR_BOARDNOTFOUND = 0x000A
ERR_BOARDINUSE = 0x000B
ERR_EXPHW64BITADR = 0x000C
ERR_FWVERSION = 0x000D
ERR_SYNCPROTOCOL = 0x000E
ERR_KERNEL = 0x000F
ERR_LASTERR = 0x0010
ERR_ABORT = 0x0020
ERR_BOARDLOCKED = 0x0030
ERR_DEVICE_MAPPING = 0x0032
ERR_NETWORKSETUP = 0x0040
ERR_NETWORKTRANSFER = 0x0041
ERR_FWPOWERCYCLE = 0x0042
ERR_NETWORKTIMEOUT = 0x0043
ERR_BUFFERSIZE = 0x0044
ERR_RESTRICTEDACCESS = 0x0045
ERR_INVALIDPARAM = 0x0046
ERR_TEMPERATURE = 0x0047
ERR_FAN = 0x0048
ERR_REG = 0x0100
ERR_VALUE = 0x0101
ERR_FEATURE = 0x0102
ERR_SEQUENCE = 0x0103
ERR_READABORT = 0x0104
ERR_NOACCESS = 0x0105
ERR_POWERDOWN = 0x0106
ERR_TIMEOUT = 0x0107
ERR_CALLTYPE = 0x0108
ERR_EXCEEDSINT32 = 0x0109
ERR_NOWRITEALLOWED = 0x010A
ERR_SETUP = 0x010B
ERR_CLOCKNOTLOCKED = 0x010C
ERR_MEMINIT = 0x010D
ERR_POWERSUPPLY = 0x010E
ERR_ADCCOMMUNICATION = 0x010F
ERR_CHANNEL = 0x0110
ERR_NOTIFYSIZE = 0x0111
ERR_TOOSMALL = 0x0112
ERR_RUNNING = 0x0120
ERR_ADJUST = 0x0130
ERR_PRETRIGGERLEN = 0x0140
ERR_DIRMISMATCH = 0x0141
ERR_POSTEXCDSEGMENT = 0x0142
ERR_SEGMENTINMEM = 0x0143
ERR_MULTIPLEPW = 0x0144
ERR_NOCHANNELPWOR = 0x0145
ERR_ANDORMASKOVRLAP = 0x0146
ERR_ANDMASKEDGE = 0x0147
ERR_ORMASKLEVEL = 0x0148
ERR_EDGEPERMOD = 0x0149
ERR_DOLEVELMINDIFF = 0x014A
ERR_STARHUBENABLE = 0x014B
ERR_PATPWSMALLEDGE = 0x014C
ERR_XMODESETUP = 0x014D
ERR_AVRG_TDA = 0x014E
ERR_NOPCI = 0x0200
ERR_PCIVERSION = 0x0201
ERR_PCINOBOARDS = 0x0202
ERR_PCICHECKSUM = 0x0203
ERR_DMALOCKED = 0x0204
ERR_MEMALLOC = 0x0205
ERR_EEPROMLOAD = 0x0206
ERR_CARDNOSUPPORT = 0x0207
ERR_CONFIGACCESS = 0x0208
ERR_FIFOBUFOVERRUN = 0x0300
ERR_FIFOHWOVERRUN = 0x0301
ERR_FIFOFINISHED = 0x0302
ERR_FIFOSETUP = 0x0309
ERR_TIMESTAMP_SYNC = 0x0310
ERR_STARHUB = 0x0320
ERR_INTERNAL_ERROR = 0xFFFF
